/*
 * Copyright (c) 2005-2012, Claybird
 * All rights reserved.

 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:

 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the Claybird nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#include "stdafx.h"
#include "ArchiverB2E.h"
#include "../Utilities/FileOperation.h"
#include "../Utilities/TemporaryDirMgr.h"
#include "../Utilities/StringUtil.h"
#include "../resource.h"
#include "../ConfigCode/ConfigExtract.h"
#include "../ConfigCode/ConfigB2E.h"

CArchiverB2E::CArchiverB2E():
	B2ESetScriptDirectory(NULL),
	B2EScriptGetCount(NULL),
	B2EScriptGetAbility(NULL),
	B2EScriptGetCompressType(NULL),
	B2EScriptGetCompressMethod(NULL),
	B2EScriptGetDefaultCompressMethod(NULL),
	B2EScriptGetExtractorIndex(NULL),
	B2EScriptGetName(NULL)
{
	m_nRequiredVersion=7;
	m_strDllName=_T("B2E32.DLL");
	m_AstrPrefix="B2E";
}

CArchiverB2E::~CArchiverB2E()
{
	FreeDLL();
}

LOAD_RESULT CArchiverB2E::LoadDLL(CConfigManager &ConfMan,CString &strErr)
{
	FreeDLL();

	//NX̃\bhĂ
	LOAD_RESULT res=CArchiverDLL::LoadDLL(ConfMan,strErr);
	if(LOAD_RESULT_OK!=res){
		return res;
	}

	//ƎAPIQ
	B2ESetScriptDirectory=(B2ESETSCRIPTDIRECTORY)GetProcAddress(m_hInstDLL,"B2ESetScriptDirectory");
	if(NULL==B2ESetScriptDirectory){
		strErr.Format(IDS_ERROR_DLL_FUNCTION_GET,m_strDllName,_T("B2ESetScriptDirectory"));
		FreeDLL();
		return LOAD_RESULT_INVALID;
	}

	B2EScriptGetCount=(B2ESCRIPTGETCOUNT)GetProcAddress(m_hInstDLL,"B2EScriptGetCount");
	if(NULL==B2EScriptGetCount){
		strErr.Format(IDS_ERROR_DLL_FUNCTION_GET,m_strDllName,_T("B2EScriptGetCount"));
		FreeDLL();
		return LOAD_RESULT_INVALID;
	}

	B2EScriptGetAbility=(B2ESCRIPTGETABILITY)GetProcAddress(m_hInstDLL,"B2EScriptGetAbility");
	if(NULL==B2EScriptGetAbility){
		strErr.Format(IDS_ERROR_DLL_FUNCTION_GET,m_strDllName,_T("B2EScriptGetAbility"));
		FreeDLL();
		return LOAD_RESULT_INVALID;
	}

	B2EScriptGetCompressType=(B2ESCRIPTGETCOMPRESSTYPE)GetProcAddress(m_hInstDLL,"B2EScriptGetCompressType");
	if(NULL==B2EScriptGetCompressType){
		strErr.Format(IDS_ERROR_DLL_FUNCTION_GET,m_strDllName,_T("B2EScriptGetCompressType"));
		FreeDLL();
		return LOAD_RESULT_INVALID;
	}

	B2EScriptGetCompressMethod=(B2ESCRIPTGETCOMPRESSMETHOD)GetProcAddress(m_hInstDLL,"B2EScriptGetCompressMethod");
	if(NULL==B2EScriptGetCompressMethod){
		strErr.Format(IDS_ERROR_DLL_FUNCTION_GET,m_strDllName,_T("B2EScriptGetCompressMethod"));
		FreeDLL();
		return LOAD_RESULT_INVALID;
	}

	B2EScriptGetDefaultCompressMethod=(B2ESCRIPTGETDEFAULTCOMPRESSMETHOD)GetProcAddress(m_hInstDLL,"B2EScriptGetDefaultCompressMethod");
	if(NULL==B2EScriptGetDefaultCompressMethod){
		strErr.Format(IDS_ERROR_DLL_FUNCTION_GET,m_strDllName,_T("B2EScriptGetDefaultCompressMethod"));
		FreeDLL();
		return LOAD_RESULT_INVALID;
	}

	B2EScriptGetExtractorIndex=(B2ESCRIPTGETEXTRACTORINDEX)GetProcAddress(m_hInstDLL,"B2EScriptGetExtractorIndex");
	if(NULL==B2EScriptGetExtractorIndex){
		strErr.Format(IDS_ERROR_DLL_FUNCTION_GET,m_strDllName,_T("B2EScriptGetExtractorIndex"));
		FreeDLL();
		return LOAD_RESULT_INVALID;
	}

	B2EScriptGetName=(B2ESCRIPTGETNAME)GetProcAddress(m_hInstDLL,"B2EScriptGetName");
	if(NULL==B2EScriptGetName){
		strErr.Format(IDS_ERROR_DLL_FUNCTION_GET,m_strDllName,_T("B2EScriptGetName"));
		FreeDLL();
		return LOAD_RESULT_INVALID;
	}

	// [h
	//===================
	// ݒ
	CConfigB2E Config;
	Config.load(ConfMan);
	if(_tcslen(Config.ScriptDirectory)>0 && PathIsDirectory(Config.ScriptDirectory)){
		B2ESetScriptDirectory(CT2A(Config.ScriptDirectory));
	}

	return LOAD_RESULT_OK;
}

void CArchiverB2E::FreeDLL()
{
	if(m_hInstDLL){
		B2ESetScriptDirectory=NULL;
		B2EScriptGetCount=NULL;
		B2EScriptGetAbility=NULL;
		B2EScriptGetCompressType=NULL;
		B2EScriptGetCompressMethod=NULL;
		B2EScriptGetDefaultCompressMethod=NULL;
		B2EScriptGetExtractorIndex=NULL;
		CArchiverDLL::FreeDLL();
	}
}

bool CArchiverB2E::Compress(LPCTSTR ArcFileName,std::list<CString> &ParamList,CConfigManager &ConfigManager,const PARAMETER_TYPE,int Options,LPCTSTR lpszFormat,LPCTSTR lpszMethod,LPCTSTR,CString &strLog)
{
	return B2ECompress(ArcFileName,ParamList,ConfigManager,lpszFormat,lpszMethod,(Options & COMPRESS_SFX) !=0 ,strLog);
}

bool CArchiverB2E::B2ECompress(LPCTSTR ArcFileName,std::list<CString> &ParamList,CConfigManager &ConfigManager,LPCTSTR lpszFormat,LPCTSTR lpszMethod,bool bSFX,CString &strLog)
{
	if(!IsOK()||!lpszFormat){
		return false;
	}

	ASSERT(0!=_tcslen(ArcFileName));
	TRACE(_T("ArcFileName=%s\n"),ArcFileName);

	//==============================================
	// X|Xt@Cpe|t@C擾
	//==============================================
	TCHAR ResponceFileName[_MAX_PATH+1];
	FILL_ZERO(ResponceFileName);
	if(!UtilGetTemporaryFileName(ResponceFileName,_T("b2e"))){
		strLog=CString(MAKEINTRESOURCE(IDS_ERROR_TEMPORARY_FILE_CREATE));
		return false;
	}
	ASSERT(0!=_tcslen(ResponceFileName));

	//==========================================
	// X|Xt@CɃA[JCu
	// kΏۃt@CL
	//==========================================
	{
		HANDLE hFile=CreateFile(ResponceFileName,GENERIC_WRITE,0,NULL,CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL);
		if(INVALID_HANDLE_VALUE==hFile){
			strLog=CString(MAKEINTRESOURCE(IDS_ERROR_TEMPORARY_FILE_ACCESS));
			return false;
		}

		TRACE(_T("X|Xt@Cւ̏\n"));
		//kt@C
		/*
		 * B2Ep̏o̓t@Cɂ́Agq.archivet̂w肳B
		 * ́Ãt@Cč폜Ȃ悤ɂ邽߂ł
		 * ŉ߂ė]vȊgq폜
		 */
		CPath tmpName=ArcFileName;
		tmpName.RemoveExtension();
		WriteResponceFile(hFile,tmpName);

		std::list<CString>::iterator ite;
		for(ite=ParamList.begin();ite!=ParamList.end();ite++){
			WriteResponceFile(hFile,*ite);
		}
		CloseHandle(hFile);
	}

	//===========================
	// DLLɓnIvV̐ݒ
	//===========================
	TRACE(_T("DLLɓnIvV̐ݒ\n"));

	CString Param;//R}hC p[^ obt@

	//---kp[^
	//k`
	Param =_T("\"-c");
	Param+=lpszFormat;
	Param+=_T("\" ");

	//k\bh
	if(lpszMethod&&_tcslen(lpszMethod)>0){
		Param+=_T("\"-m");
		Param+=lpszMethod;
		Param+=_T("\" ");
	}

	//ȉ
	if(bSFX){
		Param+=_T("-s ");
	}

	//X|Xt@Cw
	Param+=_T("\"-@");
	Param+=ResponceFileName;
	Param+=_T("\"");

	ASSERT(!Param.IsEmpty());
	TRACE(_T("ArchiveHandler Commandline Parameter:%s\n"),Param);

	TRACE(_T("ArchiveHandlerĂяo\n"));

	//---s
	std::vector<char> szLog(LOG_BUFFER_SIZE);
	szLog[0]='\0';
	int Ret=ArchiveHandler(NULL,CT2A(Param),&szLog[0],LOG_BUFFER_SIZE-1);
	strLog=&szLog[0];

	//gX|Xt@C͏
	DeleteFile(ResponceFileName);

	return 0==Ret;
}

bool CArchiverB2E::Extract(LPCTSTR ArcFileName,CConfigManager&,const CConfigExtract &Config,bool bSafeArchive,LPCTSTR OutputDir,CString &strLog)
{
	if(!IsOK()){
		return false;
	}

	if(!bSafeArchive){
		//sȃpX΁As邩ǂmF
		//ApX`FbNs\ȂƂɂ͂̌xoȂB
		CString msg;
		msg.Format(IDS_ASK_CONTINUE_EXTRACT,ArcFileName);
		if(IDYES!=MessageBox(NULL,msg,UtilGetMessageCaption(),MB_YESNO|MB_ICONQUESTION|MB_DEFBUTTON2)){
			return false;
		}
	}

	//===========================
	// DLLɓnIvV̐ݒ
	//===========================
	CString Param;//R}hC p[^ obt@

	//𓀃p[^
	Param+=_T("-e ");	//
	//㏑ɂ͖Ή

	//o͐w
	Param+=_T("\"-o");
	Param+=OutputDir;
	Param+=_T("\" ");

	//A[JCut@Cw
	Param+=_T("\"");
	Param+=ArcFileName;
	Param+=_T("\"");

	ASSERT(!Param.IsEmpty());
	TRACE(_T("ArchiveHandler Commandline Parameter:%s\n"),Param);

	TRACE(_T("ArchiveHandlerĂяo\n"));
	std::vector<char> szLog(LOG_BUFFER_SIZE);
	szLog[0]='\0';
	int Ret=ArchiveHandler(NULL,CT2A(Param),&szLog[0],LOG_BUFFER_SIZE-1);
	strLog=&szLog[0];


	return 0==Ret;
}


//=========================================================
// B2EGetFileName()̏o͌ʂɁAi[ꂽt@CpX
// Ă邩ǂʂAdtH_쐬h
//=========================================================
bool CArchiverB2E::ExamineArchive(LPCTSTR ArcFileName,CConfigManager& ConfMan,bool,bool &bInFolder,bool &bSafeArchive,CString &BaseDir,CString &strErr)
{
	if(!IsOK()){
		return false;
	}
/*	UINT uIndex=B2EScriptGetExtractorIndex(ArcFileName);
	if(uIndex==0xFFFFFFFF)return false;	//Ή`G[

	WORD wAbility;
	if(!B2EScriptGetAbility(uIndex,&wAbility))return false;
	if(!(wAbility&B2EABILITY_LIST))return false;	//łȂ*/

	//------
	//łȂƂɂ́AExamineArchive̎|񍐂Ă̂łKv͂Ȃ

	return _ExamineArchive(ArcFileName,ConfMan,bInFolder,bSafeArchive,0,BaseDir,strErr);
}

bool CArchiverB2E::ExtractSpecifiedOnly(LPCTSTR ArcFileName,CConfigManager&,LPCTSTR OutputDir,std::list<CString> &FileList,CString &strLog,bool bUsePath)
{
	if(!IsOK()){
		return false;
	}

	//==============================================
	// X|Xt@Cpe|t@C擾
	//==============================================
	TCHAR ResponceFileName[_MAX_PATH+1];
	FILL_ZERO(ResponceFileName);
	if(!UtilGetTemporaryFileName(ResponceFileName,_T("b2e"))){
		strLog=CString(MAKEINTRESOURCE(IDS_ERROR_TEMPORARY_FILE_CREATE));
		return false;
	}
	ASSERT(0!=_tcslen(ResponceFileName));

	//𓀑Ώۃt@CX|Xt@Cɏo
	{
		HANDLE hFile=CreateFile(ResponceFileName,GENERIC_WRITE,0,NULL,CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL);
		if(INVALID_HANDLE_VALUE==hFile){
			strLog=CString(MAKEINTRESOURCE(IDS_ERROR_TEMPORARY_FILE_ACCESS));
			return false;
		}

		std::list<CString>::iterator ite=FileList.begin();
		const std::list<CString>::iterator end=FileList.end();
		for(;ite!=end;ite++){
			WriteResponceFile(hFile,*ite);
		}
		CloseHandle(hFile);
	}

	//===========================
	// DLLɓnIvV̐ݒ
	//===========================
	CString Param;

	//𓀃p[^
	Param+=_T("-e ");			//
	if(!bUsePath)Param+=_T("-g ");		//pX𖳌

	//A[JCut@Cw
	Param+=_T("\"");
	Param+=ArcFileName;
	Param+=_T("\" ");

	//o͐tH_
	Param+=_T("\"-o");
	Param+=OutputDir;
	Param+=_T("\" ");

	//X|Xt@Cw
	Param+=_T("\"-@");
	Param+=ResponceFileName;
	Param+=_T("\"");

	TRACE(_T("ArchiveHandlerĂяo\nCommandline Parameter:%s\n"),Param);
	//char szLog[LOG_BUFFER_SIZE]={0};
	std::vector<char> szLog(LOG_BUFFER_SIZE);
	szLog[0]='\0';
	int Ret=ArchiveHandler(NULL,CT2A(Param),&szLog[0],LOG_BUFFER_SIZE-1);
	strLog=&szLog[0];
	//gX|Xt@C͏
	DeleteFile(ResponceFileName);

	return 0==Ret;
}

bool CArchiverB2E::QueryExtractSpecifiedOnlySupported(LPCTSTR ArcFileName)const
{
	UINT uIndex=B2EScriptGetExtractorIndex(CT2A(ArcFileName));
	if(uIndex==0xFFFFFFFF)return false;	//Ή`G[

	WORD wAbility;
	if(!B2EScriptGetAbility(uIndex,&wAbility))return false;
	if(wAbility&B2EABILITY_MELT_EACH)return true;	//ʉ𓀂łȂ
	return false;
}

bool CArchiverB2E::EnumCompressB2EScript(std::vector<B2ESCRIPTINFO> &ScriptInfoArray)
{
	ScriptInfoArray.clear();

	//XNvg̐擾
	const UINT uScriptCount=B2EScriptGetCount();

	if(uScriptCount==-1)return false;	//G[

	for(UINT uIndex=0;uIndex<uScriptCount;uIndex++){
		B2ESCRIPTINFO Info;
		Info.uIndex=uIndex;

		//\͂擾
		B2EScriptGetAbility(uIndex,&Info.wAbility);

		if(Info.wAbility&B2EABILITY_COMPRESS){	//---k\͂
			//k`擾
			B2EScriptGetCompressType(uIndex,Info.szFormat,_MAX_PATH);
			//k\bh擾
			{
				int i=0;
				char szBuffer[_MAX_PATH+1]={0};
				while(B2EScriptGetCompressMethod(uIndex,i,szBuffer,_MAX_PATH)){
					Info.MethodArray.push_back(szBuffer);
					i++;
				}
			}

			//ftHgk\bhԍ擾
			Info.nDefaultMethod=B2EScriptGetDefaultCompressMethod(uIndex);
			//---ǉ
			ScriptInfoArray.push_back(Info);
		}
	}
	return true;
}

bool CArchiverB2E::EnumActiveB2EScriptNames(std::vector<CString> &ScriptNames)
{
	ScriptNames.clear();

	//XNvg̐擾
	const UINT uScriptCount=B2EScriptGetCount();

	if(uScriptCount==-1)return false;	//G[

	for(UINT uIndex=0;uIndex<uScriptCount;uIndex++){
		B2ESCRIPTINFO Info;
		Info.uIndex=uIndex;

		//\͂擾
		B2EScriptGetAbility(uIndex,&Info.wAbility);

		if(Info.wAbility){	//---LȃXNvg
			char buffer[_MAX_PATH];
			B2EScriptGetName(uIndex,buffer,COUNTOF(buffer));
			ScriptNames.push_back(CString(buffer));
		}
	}
	return true;
}


bool CArchiverB2E::AddItemToArchive(LPCTSTR ArcFileName,const std::list<CString> &FileList,CConfigManager &ConfMan,LPCTSTR lpDestDir,CString &strLog)
{
	// X|Xt@Cpe|t@C擾
	TCHAR ResponceFileName[_MAX_PATH+1];
	FILL_ZERO(ResponceFileName);
	if(!UtilGetTemporaryFileName(ResponceFileName,_T("b2e"))){
		strLog=CString(MAKEINTRESOURCE(IDS_ERROR_TEMPORARY_FILE_CREATE));
		return false;
	}
	ASSERT(0!=_tcslen(ResponceFileName));

	//===ꎞIɃt@CRs[
	//---\ŏI_pX擾
	CPath strBasePath;
	UtilGetBaseDirectory(strBasePath,FileList);
	TRACE(_T("%s\n"),strBasePath);

	//---e|ɑΏۃt@CRs[
	//e|
	CTemporaryDirectoryManager tdm(_T("lhaf"));
	CPath strDestPath(tdm.GetDirPath());
	strDestPath+=lpDestDir;
	UtilMakeSureDirectoryPathExists(strDestPath);

	// kΏۃt@CC
	const int BasePathLength=((CString)strBasePath).GetLength();
	CString strSrcFiles;	//Rs[t@C̈ꗗ
	CString strDestFiles;	//Rs[t@C̈ꗗ
	std::list<CString>::const_iterator ite;
	for(ite=FileList.begin();ite!=FileList.end();++ite){
		//x[XpXɑ΃pX擾 : ʂłpX̕Jbg
		LPCTSTR lpSrc((LPCTSTR)(*ite)+BasePathLength);

		//葤t@Cw
		strSrcFiles+=(strBasePath+lpSrc);	//PathAppend
		strSrcFiles+=_T('|');
		//󂯑t@Cw
		strDestFiles+=strDestPath+lpSrc;
		strDestFiles+=_T('|');
	}
	strSrcFiles+=_T('|');
	strDestFiles+=_T('|');

	//'|''\0'ɕϊ
	std::vector<TCHAR> srcBuf(strSrcFiles.GetLength()+1);
	UtilMakeFilterString(strSrcFiles,&srcBuf[0],srcBuf.size());
	std::vector<TCHAR> destBuf(strDestFiles.GetLength()+1);
	UtilMakeFilterString(strDestFiles,&destBuf[0],destBuf.size());

	//t@Ce
	SHFILEOPSTRUCT fileOp={0};
	fileOp.wFunc=FO_COPY;
	fileOp.fFlags=FOF_MULTIDESTFILES|FOF_NOCONFIRMATION|FOF_NOCONFIRMMKDIR|FOF_NOCOPYSECURITYATTRIBS|FOF_NO_CONNECTED_ELEMENTS;
	fileOp.pFrom=&srcBuf[0];
	fileOp.pTo=&destBuf[0];

	//Rs[s
	if(::SHFileOperation(&fileOp)){
		//G[
		strLog=CString(MAKEINTRESOURCE(IDS_ERROR_FILE_COPY));
		return false;
	}else if(fileOp.fAnyOperationsAborted){
		//LZ
		strLog=CString(MAKEINTRESOURCE(IDS_ERROR_USERCANCEL));
		return false;
	}

	//JgfBNgݒ
	::SetCurrentDirectory(tdm.GetDirPath());
	// ɁAX|Xt@CɃA[JCuшkΏۃt@CL
	{
		HANDLE hFile=CreateFile(ResponceFileName,GENERIC_WRITE,0,NULL,CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL);
		if(INVALID_HANDLE_VALUE==hFile){
			strLog=CString(MAKEINTRESOURCE(IDS_ERROR_TEMPORARY_FILE_ACCESS));
			return false;
		}
		//X|Xt@Cւ̏
		//SĈk
		WriteResponceFile(hFile,_T("*.*"));
		CloseHandle(hFile);
	}

	//===========================
	// DLLɓnIvV̐ݒ
	//===========================
	TRACE(_T("DLLɓnIvV̐ݒ\n"));

	CString Param;//R}hC p[^ obt@

	//---kp[^
	//k`
	Param =_T("-a ");

	//A[JCut@Cw
	Param+=_T("\"");
	Param+=ArcFileName;
	Param+=_T("\" ");

	//X|Xt@Cw
	Param+=_T("\"-@");
	Param+=ResponceFileName;
	Param+=_T("\"");

	ASSERT(!Param.IsEmpty());
	TRACE(_T("ArchiveHandler Commandline Parameter:%s\n"),Param);

	//---s
	std::vector<char> szLog(LOG_BUFFER_SIZE);
	szLog[0]='\0';
	int Ret=ArchiveHandler(NULL,CT2A(Param),&szLog[0],LOG_BUFFER_SIZE-1);
	strLog=&szLog[0];

	//gX|Xt@C͏
	DeleteFile(ResponceFileName);

	return 0==Ret;
}

bool CArchiverB2E::QueryAddItemToArchiveSupported(LPCTSTR ArcFileName)const
{
	LPCTSTR lpExt=PathFindExtension(ArcFileName);
	if(!lpExt)return false;
	CString strExt=lpExt;
	strExt+=_T('.');	//".ext."̌`ɂ

	const int ScriptCount=B2EScriptGetCount();  //XNvg̐擾
	
	for(int i=0;i<ScriptCount;i++){
		WORD ability;
		B2EScriptGetAbility(i,&ability);        //\͂擾
		if(ability & B2EABILITY_ADD){
			char Buffer[_MAX_PATH+1];
			B2EScriptGetName(i,Buffer,_MAX_PATH);   //XNvg擾
			CString tmp=Buffer;
			tmp.Replace(_T("#"),_T(""));
			tmp=_T('.')+tmp;
			if(tmp.Find(strExt)!=-1){
				//Ήgq(".ext.")
				return true;
			}
		}
	}
	return false;
}

bool CArchiverB2E::DeleteItemFromArchive(LPCTSTR ArcFileName,CConfigManager &ConfMan,const std::list<CString> &FileList,CString &strLog)
{
	if(!IsOK()){
		return false;
	}

	//==============================================
	// X|Xt@Cpe|t@C擾
	//==============================================
	TCHAR ResponceFileName[_MAX_PATH+1];
	FILL_ZERO(ResponceFileName);
	if(!UtilGetTemporaryFileName(ResponceFileName,_T("b2e"))){
		strLog=CString(MAKEINTRESOURCE(IDS_ERROR_TEMPORARY_FILE_CREATE));
		return false;
	}
	ASSERT(0!=_tcslen(ResponceFileName));

	//𓀑Ώۃt@CX|Xt@Cɏo
	{
		HANDLE hFile=CreateFile(ResponceFileName,GENERIC_WRITE,0,NULL,CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL);
		if(INVALID_HANDLE_VALUE==hFile){
			strLog=CString(MAKEINTRESOURCE(IDS_ERROR_TEMPORARY_FILE_ACCESS));
			return false;
		}

		std::list<CString>::const_iterator ite=FileList.begin();
		const std::list<CString>::const_iterator end=FileList.end();
		for(;ite!=end;ite++){
			WriteResponceFile(hFile,*ite);
		}
		CloseHandle(hFile);
	}

	//===========================
	// DLLɓnIvV̐ݒ
	//===========================
	CString Param;

	//p[^
	Param+=_T("-d ");			//폜

	//A[JCut@Cw
	Param+=_T("\"");
	Param+=ArcFileName;
	Param+=_T("\" ");

	//X|Xt@Cw
	Param+=_T("\"-@");
	Param+=ResponceFileName;
	Param+=_T("\"");

	TRACE(_T("ArchiveHandlerĂяo\nCommandline Parameter:%s\n"),Param);
	//char szLog[LOG_BUFFER_SIZE]={0};
	std::vector<char> szLog(LOG_BUFFER_SIZE);
	szLog[0]='\0';
	int Ret=ArchiveHandler(NULL,CT2A(Param),&szLog[0],LOG_BUFFER_SIZE-1);
	strLog=&szLog[0];
	//gX|Xt@C͏
	DeleteFile(ResponceFileName);

	return 0==Ret;
}

bool CArchiverB2E::QueryDeleteItemFromArchiveSupported(LPCTSTR ArcFileName)const
{
	LPCTSTR lpExt=PathFindExtension(ArcFileName);
	if(!lpExt)return false;
	CString strExt=lpExt;
	strExt+=_T('.');	//".ext."̌`ɂ

	const int ScriptCount=B2EScriptGetCount();  //XNvg̐擾
	
	for(int i=0;i<ScriptCount;i++){
		WORD ability;
		B2EScriptGetAbility(i,&ability);        //\͂擾
		if(ability & B2EABILITY_DELETE){
			char Buffer[_MAX_PATH+1];
			B2EScriptGetName(i,Buffer,_MAX_PATH);   //XNvg擾
			CString tmp=Buffer;
			tmp.Replace(_T("#"),_T(""));
			tmp=_T('.')+tmp;
			if(tmp.Find(strExt)!=-1){
				//Ήgq(".ext.")
				return true;
			}
		}
	}
	return false;
}
