/*
 * Copyright (c) 2005-2012, Claybird
 * All rights reserved.

 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:

 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the Claybird nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#include "stdafx.h"
#include "ArchiverJACK.h"
#include "../resource.h"
#include "../ConfigCode/ConfigExtract.h"
#include "../ConfigCode/ConfigJACK.h"
#include "../Dialogs/JackVolumeSizeDlg.h"
#include "../Utilities/OSUtil.h"

CArchiverJACK::CArchiverJACK()
{
	m_nRequiredVersion=20;
	m_strDllName=_T("Jack32.dll");
	m_AstrPrefix="Jack";
	m_LoadLevel=LOAD_DLL_MINIMUM;
}

CArchiverJACK::~CArchiverJACK()
{
	FreeDLL();
}

bool CArchiverJACK::Compress(LPCTSTR ArcFileName,std::list<CString> &ParamList,CConfigManager &ConfMan,const PARAMETER_TYPE Type,int Options,LPCTSTR,LPCTSTR,LPCTSTR,CString &strLog)
{
	if(!IsOK()){
		return false;
	}

	//ArcFileName͏o͐tH_
	ASSERT(0!=_tcslen(ArcFileName));
	TRACE(_T("ArcFileName=%s\n"),ArcFileName);

	//===========================
	// DLLɓnIvV̐ݒ
	//===========================
	TRACE(_T("DLLɓnIvV̐ݒ\n"));

	CString Param;//R}hC p[^ obt@

	CConfigJACK Config;
	Config.load(ConfMan);
	Param+=_T("-r ");	//
	if(Options&COMPRESS_SFX){
		Param+=_T("-m1 ");	//SFX
	}
	else{
		Param+=_T("-m0 ");	//ʏ
	}
	if(Config.SpecifyVolumeSizeAtCompress){
		CJackVolumeSizeDialog vsd;
		if(IDOK!=vsd.DoModal()){
			return false;
		}
		CString Buf;
		Buf.Format(_T("-v:%d "),vsd.VolumeSize);
		Param+=Buf;//TCYw
	}else{
		CString Buf;
		Buf.Format(_T("-v:%d "),Config.VolumeSize);
		Param+=Buf;//TCYw
	}

	//Ώۃt@Cw
	Param+=_T("\"");
	Param+=*ParamList.begin();
	Param+=_T("\" ");

	//o̓tH_w
	Param+=_T("\"");
	Param+=ArcFileName;
	Param+=_T("\"");

	ASSERT(!Param.IsEmpty());
	TRACE(_T("ArchiveHandler Commandline Parameter:%s\n"),Param);

	TRACE(_T("ArchiveHandlerĂяo\n"));
	//char szLog[LOG_BUFFER_SIZE]={0};
	std::vector<char> szLog(LOG_BUFFER_SIZE);
	szLog[0]='\0';
	int Ret=ArchiveHandler(NULL,CT2A(Param),&szLog[0],LOG_BUFFER_SIZE-1);
	strLog=&szLog[0];

	return 0==Ret;
}

bool CArchiverJACK::Extract(LPCTSTR ArcFileName,CConfigManager&,const CConfigExtract &Config,bool bSafeArchive,LPCTSTR OutputDir,CString &strLog)
{
	if(!IsOK()){
		return false;
	}
	if(!bSafeArchive){
		strLog.Format(IDS_ERROR_DANGEROUS_ARCHIVE,ArcFileName);
		return false;
	}
	//o͐ړ
	CCurrentDirManager currentDir(OutputDir);

	//===========================
	// DLLɓnIvV̐ݒ
	//===========================
	TRACE(_T("DLLɓnIvV̐ݒ\n"));

	if(!Config.ForceOverwrite){//㏑mF
		CString strFileName;

		//t@CǂŊi[t@C擾
		if(!GetContainedFileName(ArcFileName,strFileName)){
			//sȃt@C:CheckArchive()ŒeĂƊ҂ł
			return false;
		}

		//--------------
		// ݃`FbN
		//--------------
		strFileName.Insert(0,OutputDir);
		if(PathFileExists(strFileName)){
			CString msg;
			msg.Format(IDS_CONFIRM_OVERWRITE_MESSAGE_SIMPLE,strFileName);
			if(IDYES!=MessageBox(NULL,msg,CString(MAKEINTRESOURCE(IDS_MESSAGE_CAPTION)),MB_YESNO|MB_ICONQUESTION)){
				return false;
			}
		}
	}


	CString Param;//R}hC p[^ obt@

	//p[^
	Param+=_T("-c ");	//

	//A[JCut@Cw
	Param+=_T("\"");
	Param+=ArcFileName;
	Param+=_T("\" ");

	//o͐w
	Param+=_T("\"");
	Param+=_T(".\\");//OutputDir;
	Param+=_T("\"");

	ASSERT(!Param.IsEmpty());
	TRACE(_T("ArchiveHandler Commandline Parameter:%s\n"),Param);

	TRACE(_T("ArchiveHandlerĂяo\n"));
	//char szLog[LOG_BUFFER_SIZE]={0};
	std::vector<char> szLog(LOG_BUFFER_SIZE);
	szLog[0]='\0';
	int Ret=ArchiveHandler(NULL,CT2A(Param),&szLog[0],LOG_BUFFER_SIZE-1);
	strLog=&szLog[0];

	return 0==Ret;
}


bool CArchiverJACK::ExtractSpecifiedOnly(LPCTSTR ArcFileName,CConfigManager&,LPCTSTR OutputDir,std::list<CString>&,CString &,bool bUsePath)
{
	return false;
}

//=========================================================
// DTV̉\̂t@CǂڊmF
//=========================================================
bool CArchiverJACK::ExamineArchive(LPCTSTR ArcFileName,CConfigManager& ConfMan,bool,bool &bInFolder,bool &bSafeArchive,CString &BaseDir,CString &strErr)
{
	bInFolder=false;
	bSafeArchive=false;

	CString strFileName;

	//t@CǂŊi[t@C擾
	if(!GetContainedFileName(ArcFileName,strFileName))return false;
	if(-1==strFileName.FindOneOf(_T(":\\/"))){//pXw̕܂܂ĂȂ̂ňS
		bSafeArchive=true;
	}

/*
	̃R[hł͉𓀂t@CAt@CemFĂȂB
	ł肪ȂŔAJACK32.dll͓WJɏo̓t@C̈ѐ`FbNĂ͗lłB
	[]
	En-1Ԗڂ܂ł̃t@Cɂ͍׍HĂȂ
		EnԖڂ̃t@C׍HĂȂ
			𓀁An++
		EnԖڂ̃t@C׍HĂ
			En-1Ԗڂ܂ł̃t@CLhaForgeɗ^
				SƂĉ𓀂n߂̂́AJAK̓ꐫ`FbNɂ
			EnԖڂ̃t@CLhaForgeɗ^
				LhaForgeDTV`FbNɂ

	[ӓ_]
	݂̃R[hł́A㏑mF@\gƂAJAKt@CQǂނƂɂȂB
	߂ȂÃR[h𖳌āAExtract()ňSǂ`FbN悤ɂ΂悢B
	܂̂Ƃ́Aʂ̗ǂD悵ĂB
*/

	return true;
}

//wb_
// Original:JakTool.cpp/XacRett #49/(C)k.inaba
int CArchiverJACK::FindHeader(const BYTE* hdr,DWORD size)
{
	static const char Magic[]="This_Is_Jack_File";

	if(size<sizeof_JakHeader)
		return -1;

	if(0==strcmp((char*)hdr,Magic)){
		if(sizeof_JakHeader+((JakHeader*)hdr)->FileNameLen<size)
			return 0;
	}
	else if(hdr[0]=='M' && hdr[1]=='Z')	//ȉ
	{
		DWORD prev = 0xffffffff;
		for( DWORD i=0; i<size-sizeof_JakHeader; i++ )
		{
			if( hdr[i]!='T' )continue;
			if( hdr[i+1]!='h' )continue;
			if( hdr[(++i)+1]!='i' )continue;
			if( hdr[(++i)+1]!='s' )continue;
			if( hdr[(++i)+1]!='_' )continue;
			if( 0!=strcmp((char*)hdr+(++i)+1,Magic+5) )continue;

			// X^u̕Ɉ邱Ƃ̂ŁA
			// "This_Is_..."Ƃ͈Skip
			if( prev==0xffffffff )
				prev = (i-4);
			else
				return (i-4);
		}
		if(prev!=0xffffffff)
			return prev;
	}
	return -1;
}

bool CArchiverJACK::GetContainedFileName(LPCTSTR ArcFileName,CString &strFileName)
{
	strFileName.Empty();

	//t@Ceobt@
	std::vector<BYTE> Buffer;
	//t@CI[v
	HANDLE hFile=CreateFile(ArcFileName,GENERIC_READ,FILE_SHARE_READ,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
	if(INVALID_HANDLE_VALUE==hFile)return false;

	DWORD dwSize=GetFileSize(hFile,NULL);
		//4GB傫ȃt@CTCY͎擾łȂAIɂ͂̂悤ȑ傫ȃt@C͈Ȃ(JACK͕p)
	Buffer.resize(dwSize);

	DWORD dwRead=0;
	if(dwSize<0){
		CloseHandle(hFile);
		return false;
	}

	//ǂݎ
	ReadFile(hFile,&Buffer[0],dwSize,&dwRead,NULL);
	CloseHandle(hFile);

	//wb_
	int iPos=FindHeader(&Buffer[0],dwRead);
	if(-1==iPos){
		//Not Found
		return false;
	}

	JakHeader* lpHeader=(JakHeader*)&Buffer[iPos];

	for(unsigned int i=0;i<lpHeader->FileNameLen;i++){
		strFileName+=Buffer[iPos+sizeof_JakHeader+i];
	}
	return true;
}
